/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_BOARDIO
#define PASANGEMAS_BOARDIO 1
#include <gtkmm/drawingarea.h>
#include <glibmm/thread.h>
#include "board.h"
#include "theme.h"

namespace Pasang {

class BoardGui : public Gtk::DrawingArea { 
  public:
    sigc::signal<void, std::string*>  next_move_signal;
    Board      board;
    Theme      theme;
    int        rotated;           // 1 if board is visually rotated 180 degree 
                                  // and bottom player moves first
  private:
    PlayerType player_type[2];    // Set by set_turn()
    enum {AT_REST=-1000};
    Point kas_point;              // Position of kas being dragged. If not dragged, x=AT_REST
    Point drag_point;             // Position of mouse pointer relative to kas_point
    bool  candidate_in;           // true if a candidate kas is dragged onto its proper position
    Move  *selected_move;
    int   frame_stop[MATRIX_SIZE];// Animate to this frame number
    int   frame[MATRIX_SIZE];     // Current frame number
    int   frame_count[MATRIX_SIZE];// Number of frames in the animation sequence
    bool  simulating_mouse;       // true if simulating mouse action for non-human player
    int   simulated_press_pos;    // Simulation: press at this position, then drag to ...
    int   simulated_release_pos;  // ... this position, and then release.
    bool  start_of_simulation;
    int   move_submission_stage;  // 0:NOP, 1:submitting, 2:waiting for the pieces to be lifted
    Glib::Timer  timer;
    double prev_time;             // Previous value when animate function was invoked (normalized 0..1)

  public:
    BoardGui ();
    virtual ~BoardGui ();
    virtual bool   on_expose_event (GdkEventExpose *event);
    virtual bool   on_motion_notify_event (GdkEventMotion* event);
    virtual bool   on_button_press_event (GdkEventButton* event);
    virtual bool   on_button_release_event (GdkEventButton* event);
    void           set_turn (PlayerType, PlayerType, int);
    PlayerType     get_player_type () const;
    PlayerSideType get_player_side () const;
    int            get_score (PlayerSideType) const;  // cmp Board::get_score(int)
    void           on_pattern_changed (const Matrix);
    void           simulate (Move&);
    void           stop_simulation ();
    void           animate ();

  private:
    bool           input_allowed ();
    void           mouse_moved (Point);
    void           mouse_pressed (Point);
    void           mouse_released (Point);
    void           queue_draw_piece (const int position);
    void           queue_draw_kas ();
    void           submit_selected_move ();
    void           perform_selected_move ();
    Move*          find_move (const Point);
    void           pick_candidate (const Point);
    void           drag_candidate (const Point);
    void           drop_candidate (const Point);
    void           pick_kas (const Point);
    void           drag_kas (const Point);
    void           drop_kas (const Point);
    void           animate_pick (const int pos, const PieceType p = PIECE_NULL);
    void           animate_pick (const Move*);
    void           animate_pick_kas ();
    void           animate_remove (const int pos, const PieceType p = PIECE_NULL);
    void           animate_remove (const Move*);
    void           animate_drop ();
    void           animate_drop_kas ();
    Point          to_point (int);
    int            to_position (const Point&);
    void           animate_simulated_mouse_events (double time);
};

}; //end namespace
#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
