/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_BOARD
#define PASANGEMAS_BOARD 1
#include "pasang-emas.h"
#include "move.h"

namespace Pasang {

class Board {
  private:
    GameStageType stage;
    Matrix        matrix;
    int           player;             // Player 0 always plays first
    int           value_of_uncaptured_pieces;
    int           score[2];
    int           kas_position[2];
    int           first_move[2];      // The first move will restrict the 2nd move.
     
  public:
    Board ();
    Board (const Board &b);
    int           count_ambiguous_forks (const Move &move) const;
    void          perform (const Move&, bool full_move);
    void          perform (const std::string& move_notation);
    PieceType     get_piece (int row, int col) const;
    PieceType     get_piece (int pos) const;
    GameStageType get_stage () const;
    int           get_player () const;
    PieceType     get_kas () const;
    int           get_first_move (int player) const;
    int           get_kas_position (int player) const;
    int           get_score (int player) const;
    void          start_game (const Matrix starting_matrix);
    
  private:
    void          perform_capture (const Move&);
    void          perform_move_kas (const Move&);
    void          change_player ();

/**
 * Move generation
 */
  private:
    enum  {MAX_BRANCH = 60};
    Move  moves[60];
    int   num_branches;
    
  public:
    void    generate_branches ();
    void    singularize_branch (Move&);
    Move&   get_branch (int n);    // Don't put const here
    int     count_branches () const;
    void    sort_branches ();
    
  private:
    void    generate_resolving_moves (const Move&);
    void    generate_opening_moves ();
    void    generate_kas_selection_moves (int distance);
    void    generate_kas_moving_moves ();
    void    add_move (int position, int capture_position, int capture_direction);
    void    add_move (int position, int num_booties, int capture_position, int capture_direction);
};

};//end namespace
#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
