/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_BRAIN
#define PASANGEMAS_BRAIN 1
#include <glibmm/thread.h>
#include <glibmm/timer.h>
#include <glibmm/random.h>
#include <glibmm/dispatcher.h>
#include "board.h"

namespace Pasang {

enum { 
  MAX_MOVE = 10,
  MAX_PLY = MAX_MOVE*2,
  MAX_TIME = 10,
  MAX_DISCERNMENT = 10
};

struct VirtualPlayer {
  int  thinking_time;       // 1 .. MAX_TIME (seconds)
  int  thinking_depth;      // 1 .. MAX_MOVE
  int  discernment;         // 1 .. MaxDiscernment. Higher => can discern "winning" moves better
  VirtualPlayer() {
    thinking_time = 1; 
    thinking_depth = 2;
    discernment = 2;
  }
};

class Brain {
  private:                        // Search controls:
    Glib::Rand    rand;
    Glib::Timer   timer;
    VirtualPlayer virtual_player;
    volatile long request_id;
    Board ai_board;
    enum  {
      POSITIVE_CUT_OFF = 10000,
      NEGATIVE_CUT_OFF = -10000 
    };
                                  // Thread controls:       
    Glib::Thread* thinking_thread;
    Glib::Mutex   mutex;
    Glib::Cond    cond;
    volatile bool exiting;        // true to kill the brain
    volatile bool stopping;       // true to abandon the current thinking
    volatile bool thinking;       // this variable is mutex-locked
  
  public:
    //sigc::signal<void, Move*, long> got_move_signal;
    Glib::Dispatcher  got_move_signal;
    long              response_id;
    Move              *response;
    
    Brain ();
    ~Brain ();
    void request_exit ();
    void request_stop ();
    void request_think (Board&, long request_id, VirtualPlayer);
    
  private:
    void  run ();
    Move* think ();
    int   search (int ply, int depth, long max_time, bool *depth_reached, int *nodes,
                  Board&, int my_cut_off, int opponent_cut_off);
    int   roll_biased_dice ();
};
}; //end namespace
#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
