/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "move.h"
using namespace Pasang;

Move::Move ()
{
  set(STAGE_MOVE_KAS, 0, -1, 0);
}

void 
Move::set(GameStageType _stage, int _player, int _id, int _position)
{
  stage = _stage;
  player = _player;
  id = _id; 
  position = _position;
  num_booties = 0;
  value = 0;
}

/**
 * Keep this routine standardized so that the strings it generates can
 * be used in text files and network play (keep dreaming).
 */
std::string 
Move::to_string () 
const
{
  g_assert (id != -1);
  g_assert (num_booties&1);  // Capture odd number of pieces
  char s[20];
  char col = 'A' - 1 + (12 - position % MATRIX_WIDTH);
  int  row = position / MATRIX_WIDTH;
  bool capture_vertical = (booty_positions[0] / MATRIX_WIDTH) != row;
  if (player == 1) {
    col = 'A'-1 + position % MATRIX_WIDTH;
    row = 12 - row;
  }
  if (stage == STAGE_OPENING) {
    sprintf (s, "%cx%d", col, value);
  }
  else if (stage == STAGE_SELECT_KAS && capture_vertical) {
    sprintf (s, "%c%dx%d*", col, row, value);
  }
  else {
    sprintf (s, "%c%dx%d", col, row, value);
  }
  return std::string (s);
}

/**
 * Add to the list of pieces captured by this move.
 */
void 
Move::add_booty (int booty_position, PieceType piece)
{
  g_assert (id != -1);
  g_assert (piece == PIECE_BLACK || piece == PIECE_WHITE);
  g_assert (num_booties < MATRIX_WIDTH);
  g_assert (booty_position >= 0 && booty_position < MATRIX_SIZE);
  booty_positions[ num_booties++ ] = booty_position;        
  value += (piece == PIECE_BLACK) ? 1 : 2;
}
    
/**
 * The number of tokens captured by this move.
 */
int 
Move::count_booties ()
const
{
  g_assert (id != -1);
  return num_booties;
}
    
/**
 * The position of the nth booty
 */
int 
Move::get_booty_position(int n)
const
{
  g_assert (id != -1);
  g_assert (n >= 0 && n < num_booties);
  return booty_positions[n];
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
