/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS
#define PASANGEMAS 1

#include <glib/gmessages.h>
#include <glib/gi18n.h>
#include <string>
#include <config.h>
#include <stdio.h>
#include <cstdlib>

/*
 * The following defs are superseded by #include <glib/gi18n.h>
 *
#include "gettext.h"
#define _(String) gettext (String)
#define N_(String) gettext_noop (String)
*/

#define THEME_ROOT_DIR       DATADIR "/" PACKAGE_TARNAME "/themes"
#define USER_THEME_ROOT_DIR  "." PACKAGE_TARNAME "/themes"

namespace Pasang {

class Board;
class BoardGui;
class Brain;
class BrainSetter;
class Move;
class MoveList;
class Window;
class Theme;
class PatternSelector;
class Recording;

/**
 * Dimensions of board matrix, including sentinnel rows and columns.
 * The actual playing area is 11 by 11.
 *
 * Visually, the board is 13x13, allowing for decorative borders
 */
const int MATRIX_WIDTH = 13;
const int MATRIX_SIZE = 13*13;

/**
 * Animation
 */
const int TICKS_PER_SEC = 30;

/**
 * Possible contents of a board.
 * When cast to int, the value is used to index to image portions.
 * PIECE_WHITE and PIECE_BLACK should actually correspond to visually 
 * black/dark and white/light pieces (like in chess).
 *
 * A int can also be cast into a PieceType to convert player number 
 * (either 0 or 1) into either PIECE_KAS_0 or PIECE_KAS_1.
 */
enum PieceType {
  PIECE_NULL = -999,          // Uninitialized
  PIECE_KAS_0 = 0,            // Kas for player 0
  PIECE_KAS_1 = 1,            // Kas for player 1
  PIECE_BLACK = 2,            // Black piece, worth 1 point
  PIECE_WHITE = 3,            // White piece, worth 2 point
  PIECE_COUNT = 4,            // Number of different piece types
  PIECE_GUARD = 5,            // Sentinnel piece, used for border and centre
  PIECE_GUIDE = 6,            // Guide placed in "Guided play"
  PIECE_EMPTY = 100           // Empty, not occupied by any piece
};
    
enum GameStageType {
  STAGE_NULL = -999,                // Uninitialized
  STAGE_OPENING = 0,                // The stages where players remove a column,
  STAGE_SELECT_KAS = 1,             // ... select a piece for kas (mobile piece),
  STAGE_RESOLVE_KAS_SELECTION = 2,  // ... disambiguate (which line to capture),
  STAGE_MOVE_KAS = 3,               // ... move kas,
  STAGE_RESOLVE = 4,                // ... disambiguate (which to capture, blacks or whites).
  STAGE_GAME_OVER = 5
};

typedef PieceType Matrix[MATRIX_SIZE];

enum PlayerType {
  PLAYER_VIRTUAL,
  PLAYER_HUMAN
};  

enum PlayerSideType {
  PLAYER_SIDE_TOP = 0,
  PLAYER_SIDE_BOTTOM = 1
};

struct Point {
  int x,y;
  Point() { x=y=0; }
  Point(int _x, int _y) { x=_x; y=_y; }
};

};//end namespace

#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
