/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pattern.h"
#include "ucompose.hpp"
#include <gtkmm/drawingarea.h>
#include <gtkmm/box.h>
#include <gtkmm/style.h>
#include <gtkmm/image.h>
#include <gtkmm/adjustment.h>
using namespace Pasang;

struct Pattern {
  const char *name;
  const char *dots;
}
patterns[] = {
  // Codes for symmetry type in a quadrant: 
  // R : totally random
  // T : Reflection on x=3
  // E : Reflection on y=3
  // L : Reflection on y=6-x 
  // J : Reflection on y=x
  // S : 180 rotation about (3,3)
  // H : T and E
  // X : L and J
  // F : 90 rotation about (3,3)
  // O : H and X and F
  // * : Wild card
  //
  // Codes for the second quadrant:
  // If the pattern in the second quadrant is independent of the first,
  // then the same capital letters as above are used.
  // If the pattern in the second quadrant is a copy of the first,
  // then use small letters (see figure below).
  //
  // 0 +----- 
  // 1 |\ :f/      Effect
  // 2 |i\:/t  i : translate horizontally
  // 3 |--:--  t : reflection about axes
  // 4 | /:\s  f : rotation about (0,0)
  // 5 |/ : \  s : translate vertically
  //   012345  ? : wild card
        
 {
  // Random pattern
  N_("Gardens"),
  " J J J O X OfJf"
 },
 {// Random pattern
  N_("Day and Night"),
  " d"
 },
 {// Random pattern
  N_("Fans"),
  " R R R * *fRf"
 },
 {// Random pattern
  N_("Folds"),
  " Rt Rt Rt *t RtJf JfRt"
 },
 {// Random pattern
  N_("Oblique Folds"),
  " JL JL JL JLJf JfJL"
 }, 
 {// Random pattern
  N_("Whatever"),
  " RR RR RR *? ** *?*? d"
 },

 {// Pattern name. Don't translate.
  N_("Awan Mengandung Hujan"),
  "PPHPHPPHHHP"
  "HPPHPHPPHPP"
  "HHHPHPHHHPH"
  "HPHHHPPHPHP"
  "PPHPPHPHHPH"
  "PHPPH-HPPHP"
  "HPHHPHPPHPP"
  "PHPHPPHHHPH"
  "HPHHHPHPHHH"
  "PPHPPHPHPPH"
  "PHHHPPHPHPP"
 },
 {// Pattern name. Don't translate.
  N_("Ajong Dengan Sauhnya"),
  "HPHHPPPHHPH"
  "PPHPPHPPHPP"
  "HHPHHPHHPHH"
  "HPHPHPHPHPH"
  "PPHHPHPHHPP"
  "PHPPH-HPPHP"
  "PPHHPHPHHPP"
  "HPHPHPHPHPH"
  "HHPHHPHHPHH"
  "PPHPPHPPHPP"
  "HPHHPPPHHPH"
 }, 
 {// Pattern name. Don't translate.
  N_("Awan Berturus"),
  "PHPPHHHPPHP"
  "HPHPPPPPHPH"
  "PHPHHPHHPHP"
  "PPHPHHHPHPP"
  "HPHHPHPHHPH"
  "HPPHH-HHPPH"
  "HPHHPHPHHPH"
  "PPHPHHHPHPP"
  "PHPHHPHHPHP"
  "HPHPPPPPHPH"
  "PHPPHHHPPHP"
 }, 
 {// Pattern name. Don't translate.
  N_("Awan Tunjung"),
  "PHHPPHPPHPP"
  "PPHHPHPHPPH"
  "HPPHHPHHPHH"
  "PHHHHPPHHHP"
  "PPHPPHPHHPP"
  "HHPPH-HPPHH"
  "PPHHPHPPHPP"
  "PHHHPPHHHHP"
  "HHPHHPHHPPH"
  "HPPHPHPHHPP"
  "PPHPPHPPHHP"
 }, 
 {// Pattern name. Don't translate.
  N_("Bunga Sepapan"),
  "PPHPPHPPHPP"
  "PHHHPHPHHHP"
  "HHPPHHHPPHH"
  "PHPHHPPHPHP"
  "PPHPPHPHHPP"
  "HHHPH-HPHHH"
  "PPHHPHPPHPP"
  "PHPHPPHHPHP"
  "HHPPHHHPPHH"
  "PHHHPHPHHHP"
  "PPHPPHPPHPP"
 },
 {// Pattern name. Don't translate.
  N_("Bunga Bercucuk"),
  "PPHHHHHHHPP"
  "PPPHHPHHPPP"
  "HPPPHPHPPPH"
  "HHPPPHPPPHH"
  "HHHPPHPPHHH"
  "HPPHH-HHPPH"
  "HHHPPHPPHHH"
  "HHPPPHPPPHH"
  "HPPPHPHPPPH"
  "PPPHHPHHPPP"
  "PPHHHHHHHPP"
 },
 {// Pattern name. Don't translate.
  N_("Bangkawat Kudung"),
  "PPHHPPHHHHP"
  "HPPHHPPHHPP"
  "HHPPHHPHPPH"
  "HHHPPHHPPHH"
  "HPPHPPPPHHP"
  "PPHHP-PHHPP"
  "PHHPPPPHPPH"
  "HHPPHHPPHHH"
  "HPPHPHHPPHH"
  "PPHHPPHHPPH"
  "PHHHHPPHHPP"
 },
 {// Pattern name. Don't translate.
  N_("Bangkawat / Awan Menurus"),
  "PHHPPHHPPHP"
  "HHPPHHPPHHH"
  "HPPHHPPHPHP"
  "PPHHPPHHHPP"
  "PHHPPHPHPPH"
  "HHPPH-HPPHH"
  "HPPHPHPPHHP"
  "PPHHHPPHHPP"
  "PHPHPPHHPPH"
  "HHHPPHHPPHH"
  "PHPPHHPPHHP"
 },
 {// Pattern name. Don't translate.
  N_("Ikan Bernang"),
  "PHHHPPPHHHP"
  "HPPHHPHHPPH"
  "HPPPHPHPPPH"
  "HHPPHPHPPHH"
  "PHHHPHPHHHP"
  "PPPPH-HPPPP"
  "PHHHPHPHHHP"
  "HHPPHPHPPHH"
  "HPPPHPHPPPH"
  "HPPHHPHHPPH"
  "PHHHPPPHHHP"
 }, 
 {// Pattern name. Don't translate.
  N_("Kina Putar"),
  "HPPHHPPHPHH"
  "HHPPHHPPHHP"
  "PHHPPHHPHPP"
  "HPPHPPHHPPH"
  "PPHHHPHPPHH"
  "PHHPP-PPHHP"
  "HHPPHPHHHPP"
  "HPPHHPPHPPH"
  "PPHPHHPPHHP"
  "PHHPPHHPPHH"
  "HHPHPPHHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Kalajangking 1"),
  "HPHPHPHPHPH"
  "PPHPPHPPHPP"
  "HHPHHPHHPHH"
  "PPHPPHPPHPP"
  "HPHPHHHPHPH"
  "PHPHH-HHPHP"
  "HPHPHHHPHPH"
  "PPHPPHPPHPP"
  "HHPHHPHHPHH"
  "PPHPPHPPHPP"
  "HPHPHPHPHPH"
 },
 {// Pattern name. Don't translate.
  N_("Kalajangking 2"),
  "HPHHPHPHHPH"
  "PPHPPHPPHPP"
  "HHPHHPHHPHH"
  "HPHPPHPPHPH"
  "PPHPHPHPHPP"
  "HHPHP-PHPHH"
  "PPHPHPHPHPP"
  "HPHPPHPPHPH"
  "HHPHHPHHPHH"
  "PPHPPHPPHPP"
  "HPHHPHPHHPH"
 },
 {// Pattern name. Don't translate.
  N_("Kalabutan"),
  "HPPHHPHHPPH"
  "PPHPHHHPHPP"
  "PHPHPHPHPHP"
  "HPHPPHPPHPH"
  "HHPPHPHPPHH"
  "PHHHP-PHHHP"
  "HHPPHPHPPHH"
  "HPHPPHPPHPH"
  "PHPHPHPHPHP"
  "PPHPHHHPHPP"
  "HPPHHPHHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Kalabutan Berdarah Hitam"),
  "HHPPHPHPPHH"
  "HPHHPHPHHPH"
  "PHPPHPHPPHP"
  "PHPPHPHPPHP"
  "HPHHPHPHHPH"
  "PHPPH-HPPHP"
  "HPHHPHPHHPH"
  "PHPPHPHPPHP"
  "PHPPHPHPPHP"
  "HPHHPHPHHPH"
  "HHPPHPHPPHH"
 },
 {// Pattern name. Don't translate.
  N_("Kedudukan Dian"),
  "PHPPHPHPPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "PPHPHHHPHPP"
  "HHPHPHPHPHH"
  "PPHHH-HHHPP"
  "HHPHPHPHPHH"
  "PPHPHHHPHPP"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPPHPHPPHP"
 },
 {// Pattern name. Don't translate.
  N_("Mahligai Bergantung Bertali"),
  "PHPPHPHPPHP"

  "HHHPPHPPHHH"
  "PHPHPHPHPHP"
  "PPHHHPHHHPP"
  "HPPHPHPHPPH"
  "PHHPH-HPHHP"
  "HPPHPHPHPPH"
  "PPHHHPHHHPP"
  "PHPHPHPHPHP"
  "HHHPPHPPHHH"
  "PHPPHPHPPHP"
 },
 {// Pattern name. Don't translate.
  N_("Peria Bulat"),
  "PPHHHPHHHPP"
  "PPPHHPHHPPP"
  "HPPPHHHPPPH"
  "HHPPPHPPPHH"
  "HHHPPHPPHHH"
  "PPHHH-HHHPP"
  "HHHPPHPPHHH"
  "HHPPPHPPPHH"
  "HPPPHHHPPPH"
  "PPPHHPHHPPP"
  "PPHHHPHHHPP"
 },
 {// Pattern name. Don't translate.
  N_("Peria Buruk Hatinya"),
  "HPPHPHPHPPH"
  "PHPHHPHHPHP"
  "PPHPPHPPHPP"
  "HHPHPHPHPHH"
  "PHPPHHHPPHP"
  "HPHHH-HHHPH"
  "PHPPHHHPPHP"
  "HHPHPHPHPHH"
  "PPHPPHPPHPP"
  "PHPHHPHHPHP"
  "HPPHPHPHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Paita"),
  "HPPHPHPHPPH"
  "PPHHHPHHHPP"
  "PHHPPHPPHHP"
  "HHPPPHPPPHH"
  "PHPPHHHPPHP"
  "HPHHH-HHHPH"
  "PHPPHHHPPHP"
  "HHPPPHPPPHH"
  "PHHPPHPPHHP"
  "PPHHHPHHHPP"
  "HPPHPHPHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Peria Buruk Sepanggal"),
  "HPPHPPPHHPH"
  "PPPHHHHHPPP"
  "HPHHHPPHHPP"
  "HHHPHPPPHHH"
  "PHPPPHPHHHP"
  "PHPPH-HPPHP"
  "PHHHPHPPPHP"
  "HHHPPPHPHHH"
  "PPHHPPHHHPH"
  "PPPHHHHHPPP"
  "HPHHPPPHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Paha Ayam"),
  "PPPPPHHHHHP"
  "HPPPPHHHHPP"
  "HHPPPHHHPPP"
  "HHHPPHHPPPP"
  "HHHHPHPPPPP"
  "HHHHH-HHHHH"
  "PPPPPHPHHHH"
  "PPPPHHPPHHH"
  "PPPHHHPPPHH"
  "PPHHHHPPPPH"
  "PHHHHHPPPPP"
 },
 {// Pattern name. Don't translate.
  N_("Patah-Patah"),
  "HPHPPHPPHPH"
  "PPHPHHHPHPP"
  "HHHPHPHPHHH"
  "PPPHPHPHPPP"
  "PHHPPHPPHHP"
  "HHPHH-HHPHH"
  "PHHPPHPPHHP"
  "PPPHPHPHPPP"
  "HHHPHPHPHHH"
  "PPHPHHHPHPP"
  "HPHPPHPPHPH"
 },
 {// Pattern name. Don't translate.
  N_("Peria Sejambangan"),
  "HPPHPHPHPPH"
  "PPHHHPHHHPP"
  "PHPPHPHPPHP"
  "HHPHPHPHPHH"
  "PHHPPHPPHHP"
  "HPPHH-HHPPH"
  "PHHPPHPPHHP"
  "HHPHPHPHPHH"
  "PHPPHPHPPHP"
  "PPHHHPHHHPP"
  "HPPHPHPHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Si Pugut Menunggu Beringin Buruk Hatinya"),
  "HPHPPHPPHPH"
  "PHPHHPHHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "PHHPPHPPHHP"
  "HPPHH-HHPPH"
  "PHHPPHPPHHP"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHHPHHPHP"
  "HPHPPHPPHPH"
 },
 {// Pattern name. Don't translate.
  N_("Sisir Sepapan"),
  "HPPHHPPHPPH"
  "PPHHPPHHHPP"
  "PHPHPHHPPHP"
  "HHPPHHPPHHH"
  "PHHPPHPHPPH"
  "PPHHH-HHHPP"
  "HPPHPHPPHHP"
  "HHHPPHHPPHH"
  "PHPPHHPHPHP"
  "PPHHHPPHHPP"
  "HPPHPPHHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Sarimbangun"),
  "HPPHPHPHPPH"
  "PPHPPHPPHPP"
  "PHPHHHHHPHP"
  "HPHPPHPPHPH"
  "PPHPHHHPHPP"
  "HHHHH-HHHHH"
  "PPHPHHHPHPP"
  "HPHPPHPPHPH"
  "PHPHHHHHPHP"
  "PPHPPHPPHPP"
  "HPPHPHPHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Saputangan Jatuh Ke Laut"),
  "HPPHHPHHPPH"
  "PPHHPHPHHPP"
  "PHPPHPHPPHP"
  "HHPHHPPHPHH"
  "HPHPPHPHHPH"
  "PHPPH-HPPHP"
  "HPHHPHPPHPH"
  "HHPHPPHHPHH"
  "PHPPHPHPPHP"
  "PPHHPHPHHPP"
  "HPPHHPHHPPH"
 },
 {// Pattern name. Don't translate.
  N_("Selidan Putar"),
  "PHHPHPPHHPP"
  "PPHHPPHHPPH"
  "HPPHPHHPPHH"
  "HHPPHHPPHHP"
  "PHHPPHPHPPH"
  "PPHHH-HHHPP"
  "HPPHPHPPHHP"
  "PHHPPHHPPHH"
  "HHPPHHPHPPH"
  "HPPHHPPHHPP"
  "PPHHPPHPHHP"
 },
 {// Pattern name. Don't translate.
  N_("Tikus Naik Ke Durong"),
  "HPHHPPPHHPH"
  "PPPHPHPHPPP"
  "HPPHHPHHPPH"
  "HHHPHPHPHHH"
  "PPHHPHPHHPP"
  "PHPPH-HPPHP"
  "PPHHPHPHHPP"
  "HHHPHPHPHHH"
  "HPPHHPHHPPH"
  "PPPHPHPHPPP"
  "HPHHPPPHHPH"
 },
 {// Pattern name. Don't translate.
  N_("Talapok Sepapan"),
  "HPHHHPHHHPH"
  "PPPHPPPHPPP"
  "HPHPHPHPHPH"
  "HHPPPHPPPHH"
  "HPHPHHHPHPH"
  "PPPHH-HHPPP"
  "HPHPHHHPHPH"
  "HHPPPHPPPHH"
  "HPHPHPHPHPH"
  "PPPHPPPHPPP"
  "HPHHHPHHHPH"
 },
 {// Pattern name. Don't translate.
  N_("Talapok Berantai"),
  "HPHHHPHHHPH"
  "PPPHPHPHPPP"
  "HPHPHPHPHPH"
  "HHPHPPPHPHH"
  "HPHPHPHPHPH"
  "PHPPP-PPPHP"
  "HPHPHPHPHPH"
  "HHPHPPPHPHH"
  "HPHPHPHPHPH"
  "PPPHPHPHPPP"
  "HPHHHPHHHPH"
 },
 {// Pattern name. Don't translate.
  N_("Tekuyong Naik Pasang"),
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "HPHPPHPPHPH"
  "PHPPHHHPPHP"
  "HPHHH-HHHPH"
  "PHPPHHHPPHP"
  "HPHPPHPPHPH"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
 },
 {// Pattern name. Don't translate.
  N_("Pagar Batu"),
  "PHHPPHPPHHP"
  "HPPHHPHHPPH"
  "HPPHPHPHPPH"
  "PHHPHPHPHHP"
  "PHPHPHPHPHP"
  "HPHPH-HPHPH"
  "PHPHPHPHPHP"
  "PHHPHPHPHHP"
  "HPPHPHPHPPH"
  "HPPHHPHHPPH"
  "PHHPPHPPHHP"
 },
 {// Pattern name. Don't translate.
  N_("Bintang Terhambur"),
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHP-PHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
  "PHPHPHPHPHP"
  "HPHPHPHPHPH"
 },
 {// Pattern name. Don't translate.
  N_("Bunga Kembang Berhujan"),
  "PPHPPHPPHHP"
  "HPHHPHHPHPP"
  "HHPPHHPHPHH"
  "PPHHPPHHPHP"
  "PHPHHPHPHPP"
  "HHHPP-PPHHH"
  "PPHPHPHHPHP"
  "PHPHHPPHHPP"
  "HHPHPHHPPHH"
  "PPHPHHPHHPH"
  "PHHPPHPPHPP"
 },
 {// Pattern name. Don't translate.
  N_("Lungun Sipugut"),
  "PPHPHPHPHPP"
  "PPHHPHPHHPP"
  "HHPHHPHHPHH"
  "PHHPPHPPHHP"
  "HPHPHPHPHPH"
  "PHPHP-PHPHP"
  "HPHPHPHPHPH"
  "PHHPPHPPHHP"
  "HHPHHPHHPHH"
  "PPHHPHPHHPP"
  "PPHPHPHPHPP"
 },
 {// Pattern name. Don't translate.
  N_("Galap Madam"),
  "PPPPPHHHHHP"
  "HHHHHPPPPHP"
  "HPPPPHHHPHP"
  "HPHHHPPHPHP"
  "HPHPPHPHPHP"
  "HPHPH-HPHPH"
  "PHPHPHPPHPH"
  "PHPHPPHHHPH"
  "PHPHHHPPPPH"
  "PHPPPPHHHHH"
  "PHHHHHPPPPP"
 },
 {// Pattern name. Don't translate.
  N_("Lasung-Lasung"),
  "HPPPPHHHHPH"
  "PHHHHPPPHHP"
  "HHPPPHHPPHP"
  "HPPHHPHHPHP"
  "HPHHPPPHPHP"
  "HPHPP-PPHPH"
  "PHPHPPPHHPH"
  "PHPHHPHHPPH"
  "PHPPHHPPPHH"
  "PHHPPPHHHHP"
  "HPHHHHPPPPH"
 },
 {// Pattern name. Don't translate.
  N_("Tudung Dulang"),
  "PHPHPHPHPHP"
  "HPPHPHPHPPH"
  "PPHHPHPHHPP"
  "HHHPPHPPHHH"
  "PPPPHHHPPPP"
  "HHHHH-HHHHH"
  "PPPPHHHPPPP"
  "HHHPPHPPHHH"
  "PPHHPHPHHPP"
  "HPPHPHPHPPH"
  "PHPHPHPHPHP"
 },
 {// Pattern name. Don't translate.
  N_("Mayang Piasau"),
  "PHHPPHHHPPP"
  "PHPHPPPHPHH"
  "PPHHPHPHHPH"
  "HHHPHPHPHHP"
  "HPPHPHPHPPP"
  "HPHPH-HPHPH"
  "PPPHPHPHPPH"
  "PHHPHPHPHHH"
  "HPHHPHPHHPP"
  "HHPHPPPHPHP"
  "PPPHHHPPHHP"
 },
 {NULL}
};

/**
 * Translate ASCII pattern to internal format
 */

char
PatternSelector::rand_char (const char *s) {
  return s[rand.get_int_range (0, strlen (s))];
}

char
PatternSelector::read_code (char code) {
  char c = code == '*' ? rand_char ("RTELJSHXFO") :
           code == '?' ? rand_char ("ifts") :
                         code ;
  g_assert (strchr ("RTELJSHXFOifts", c));
  return c;
}

void
PatternSelector::build_random_day_and_night (int pattern_num) {
  int mode = rand.get_int_range (0,2);
  char* dots = random_dots[pattern_num];
  strcpy (dots, mode == 0 ?
                  "PPPPPHHHHHP"
                  "HPPPPHHHHPP"
                  "HHPPPHHHPPP"
                  "HHHPPHHPPPP"
                  "HHHHPHPPPPP"
                  "HHHHH-HHHHH"
                  "PPPPPHPHHHH"
                  "PPPPHHPPHHH"
                  "PPPHHHPPPHH"
                  "PPHHHHPPPPH"
                  "PHHHHHPPPPP"
                :
                  "PPPPPHHHPPP"
                  "PPPPPHHHPPP"
                  "PPPPPHHHPPP"
                  "HHHHHHHHPPP"
                  "HHHHHHHHPPP"
                  "HHHHH-HHHHH"
                  "PPPHHHHHHHH"
                  "PPPHHHHHHHH"
                  "PPPHHHPPPPP"
                  "PPPHHHPPPPP"
                  "PPPHHHPPPPP"
                  );
  for (int n=rand.get_int_range(0, 20); n > 0; n--) {
    int x[4], y[4];
    x[0] = rand.get_int_range (1, 6);
    y[0] = rand.get_int_range (0, 6);
    x[1] = mode == 0 ? 6 - x[0] : x[0];
    y[1] = 5 - y[0];
    x[2] = -y[0];
    y[2] = x[0];
    x[3] = -y[1];
    y[3] = x[1];
    for (int i=0; i < 4; i++) {
      int xy = (x[i] + 5) + 11 * (y[i] + 5);
      dots[xy] = 'H' + 'P' - dots[xy];
      dots[120 - xy] = 'H' + 'P' - dots[120 - xy];
    }
  }
}

void
PatternSelector::build_random_dots (int pattern_num) {
  g_assert (is_randomly_generated (pattern_num));
  const char *code_list = patterns[pattern_num].dots;
  const char *code;        // A random position containing a space
  do {
    code = code_list + rand.get_int_range (0, strlen (code_list));
  } while ( *code != ' ');
  code++;            // Skip the space    

  if (*code == 'd') {
    build_random_day_and_night (pattern_num);
    return;
  }
  
  // Construct implicit code. e.g., O => OfOf,  Sft => SfSt
  char code1 = read_code (*code++);
  char code2 = *code && *code != ' '   ?  read_code (*code++) : 'f';
  char code3 = *code && *code != ' '   ?  read_code (*code++) : code1;
  char code4 = *code && *code != ' '   ?  read_code (*code++) : code2;
  char *dots = random_dots[pattern_num];

  for (int attempt=0; ; attempt++) { // break when we get exactly 60 whites and 60 blacks
    for (int i=0; i <= 120; i++) {
      dots[i] = 'H';
    }
    dots[60] = '-';
    
    int center_range = rand.get_int_range (2*2, 4*4);
    int stack[40];
    int num_whites = 0;
    while (num_whites < 60) {
      int x = rand.get_int_range (0, 6);
      int y = rand.get_int_range (0, 6);
      if (x == 0 && y == 0) continue;
      bool center =  x*x + y*y <= center_range;
      char orig_type = center? code3 : code1;
      char copy_type = center? code4 : code2;
      bool duplicate_tile = copy_type >= 'a' && copy_type <= 'z';
      bool second_quarter = !duplicate_tile && (rand.get_int_range (0,2) == 1);
      if (second_quarter) {
        orig_type = copy_type;
        copy_type = 'i';
      }
      
      int sp = 0;      
      stack[sp++] =   x   + 100 * y;                    // Store (x,y) as 100y+x
      if (strchr ("JXO", orig_type))
        stack[sp++] = y   + x * 100;                    // Reflect on y = x
      if (strchr ("FO",  orig_type))
        stack[sp++] = 6-y + 100 * x;                    // Reflect J on x = 3
      if (strchr ("THO", orig_type))
        stack[sp++] = 6-x + 100 * y;                    // Reflect on x = 3
      if (strchr ("SXHFO", orig_type))
        stack[sp++] = 6-x + 100 * (6-y);                // Rotate about (3,3)
      if (strchr ("LXO", orig_type))
        stack[sp++] = 6-y + 100 * (6-x);                // Rotate J about (3,3)
      if (strchr ("FO",  orig_type))
        stack[sp++] = y   + 100 * (6-x);                // Reflect J on y = 3
      if (strchr ("EHO", orig_type))
        stack[sp++] = x   + 100 * (6-y);                // Reflect on y = 3
      
      int stack_base = sp;
      for (int i=0; i < stack_base; i++) {
        int xy = stack[i];
        int x = xy % 100;
        int y = xy / 100;
        g_assert (x >= 0 && y >= 0);
        if (x > 5 || y > 5) continue;
        if (!second_quarter) {
          // Confirm in 1st quarter (now storing the value as actual board position)
          stack[sp++] = (x + 5) + 11 * (y + 5);
          
        }
        if (!duplicate_tile && !second_quarter) continue;
        xy =  copy_type == 'f' ? (y == 0 || x == 0) ?
                                  y   + 100 * x :       // Reflect on y = x
                                  6-y + 100 * x :       // Reflect J on x = 3
              copy_type == 't' ? 6-x + 100 * y :        // Reflect on x = 3
              copy_type == 's' ? 6-x + 100 * (6-y) :    // Rotate about (3,3)
                                  x   + 100 * y;
        x = xy % 100;
        y = xy / 100;
        int cut_off = second_quarter ? 1 : 0;
        if (x >= cut_off && y >= cut_off && x <= 5 && y <= 5) {
          // Copy onto 2nd quarter (now storing the value as actual board position)
          stack[sp++] = (x + 5) + 11 * (y + 5) + (x == 0 || y == 0 ? 0 : -6);
        }
      }
        
      for (int i=stack_base; i < sp; i++) {
        int xy = stack[i];
        int x = xy % 11 - 5;
        int y = xy / 11 - 5;
        if (center && x*x + y*y <= center_range || 
            !center && x*x + y*y > center_range  || 
            code1 == code3 && code2 == code4) {
          g_assert (dots[xy] == 'P' || dots[xy] == 'H');
          g_assert (dots[xy] == dots[120 - xy]);  // Symmetry must be maintained
          if (dots[xy] == 'H') {
            dots[xy] =
            dots[120 - xy] = 'P';
            num_whites += 2;
          }
        }
      }
    }//endwhile num_whites
    
    if (num_whites > 60) {
      // Invalid arrangement. This is not fatal.
      continue;
    }
    else {
      //printf("Attempt #%d at symmetry: %c%c%c%c \n", attempt, code1, code2, code3, code4);
      break;
    }
  }//endwhile attempt  
}

void 
PatternSelector::get_matrix (Matrix matrix, int pattern_num)
{
  const char *dots = is_randomly_generated (pattern_num) ? random_dots[pattern_num] : 
                                                     patterns[pattern_num].dots;
  for (int row=0; row < MATRIX_WIDTH; row++) {
    for (int col=0; col < MATRIX_WIDTH; col++) {
      const int n = row * MATRIX_WIDTH + col;
      if (row >= 1 && row <= 11 && col >= 1 && col <= 11) {
        const char c = *dots++;
        matrix[n] = c == 'P' ? PIECE_WHITE :
                    c == 'H' ? PIECE_BLACK :
                               PIECE_GUARD;
      }
      else {
        matrix[n] = PIECE_GUARD;
      }
    }//endfor col
  }//endfor row
}


PatternSelector::PatternSelector ()
{
  last_pattern_num = 0;
  num_patterns = 0;
  ignore_signal = false; 
  sensitive = false; 
  hot = false; 
  self_clicked = false;
  pattern_data = NULL;
  for (int i=0; patterns[i].name; i++) {
    num_patterns++;
  }
  // Add another invisible button to support unselect()
  pattern_buttons = new Gtk::RadioButton*[num_patterns + 1];
  Gtk::VBox* pattern_box = Gtk::manage (new Gtk::VBox);
  add (*pattern_box);
  Gtk::RadioButtonGroup group;
  for (int i=0; i < num_patterns + 1; i++) {
    pattern_buttons[i] = new Gtk::RadioButton (); 
    if (i == 0) {
      group = pattern_buttons[i]->get_group ();
    }
    else {
      pattern_buttons[i]->set_group (group);
    }
    if (i == num_patterns) break; // The invisible button
    if (is_randomly_generated (i)) {
      hot = false;
      build_random_dots (i);
    }
    Gtk::Image *pattern_image = new Gtk::Image (create_pixbuf (i));
    pattern_buttons[i]->set_mode (false);
    pattern_buttons[i]->set_relief (Gtk::RELIEF_NONE);
    pattern_buttons[i]->set_image (*pattern_image);
    // Use signal_clicked rather than signal_toggled to allow [Random] pattern
    // to be clicked again (despite being already selected)
    pattern_buttons[i]->signal_clicked().connect 
      (sigc::bind (sigc::mem_fun (*this, &PatternSelector::on_selection), i));
    pattern_box->add (*(pattern_buttons[i]));
  }
  set_policy (Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);
}


Glib::RefPtr<Gdk::Pixbuf> 
PatternSelector::create_pixbuf (int pattern_num) 
{
  Matrix matrix;
  get_matrix (matrix, pattern_num);
  return create_pixbuf (pattern_num, matrix);
}

Glib::RefPtr<Gdk::Pixbuf> 
PatternSelector::create_pixbuf (int pattern_num, Matrix matrix) 
{
  Glib::RefPtr<Gtk::Style> style = get_style ();
  Gdk::Color pattern_color = style->get_fg (Gtk::STATE_NORMAL); 
  guint red   = pattern_color.get_red ();
  guint green = pattern_color.get_green ();
  guint blue  = pattern_color.get_blue ();
  
  // size in pixels
  const int dot_width       = 4;
  const int dot_cut         = 4;   
  const int image_width     = MATRIX_WIDTH * dot_width;
  // size in bytes (in-memory representation)
  const int sizeof_pixel    = 4;                   
  const int sizeof_dot      = dot_width * dot_width * sizeof_pixel;
  const int sizeof_image    = image_width * image_width * sizeof_dot;
                              // buggy previous version: image_width * image_width * sizeof_pixel;
  if (pattern_data == NULL) {
    pattern_data = new guint8[sizeof_image * num_patterns];
  }
  
  // Temperature for random pattern buttons
  double temperature = hot ? 3.0 - MIN (MAX (timer.elapsed () / 3, 2.0), 3.0) : 0;
  if (temperature == 0) hot = false;
  
  guint8 *data = &pattern_data[pattern_num * sizeof_image];

  for (int i=0; i < MATRIX_SIZE; i++) {
    const int dot_ndx = i / MATRIX_WIDTH * MATRIX_WIDTH * sizeof_dot
                      + i % MATRIX_WIDTH * dot_width * sizeof_pixel;
    
    for (int px=0; px < dot_width; px++) {
      for (int py=0; py < dot_width; py++) {
        const int n = dot_ndx  +  py * image_width * sizeof_pixel  +  px * sizeof_pixel;
        if (matrix[i] == PIECE_WHITE && px%dot_cut != dot_cut-1) {
          // The pattern weaved in gold
          data[n + 0] = 
          data[n + 1] = 155 + 100 / (py%dot_cut  + 1);
          data[n + 2] = 55 + 100 / (py%dot_cut + 1);;
          data[n + 3] = 255;
        }
        else if (is_randomly_generated (pattern_num)) {
          // Background cloth for random patterns
          data[n + 0] = (int)((255 - 100 / (3 - (px+py)%3)) * temperature);
          data[n + 1] = (int)((150 - 100 / (3 - (px+py)%3)) * (1 - temperature/2));
          data[n + 2] = 0;
          data[n + 3] = 255;
        }
        else {
          // Background cloth for traditional patterns
          data[n + 0] = 
          data[n + 1] = 
          data[n + 2] = 100 - 100 / (3 - (px+py)%3);
          data[n + 3] = 255;
        }
      }//endfor py
    }//endfor px
  }//endfor i

  return Gdk::Pixbuf::create_from_data 
    ( data, 
      Gdk::COLORSPACE_RGB, 
      true,                     // has alpha
      8,                        // bits per sample
      image_width,
      image_width,
      image_width * sizeof_pixel );
}


PatternSelector::~PatternSelector () {
  for (int i=0; i < num_patterns; i++) {
    delete pattern_buttons[i]->get_image ();
    delete pattern_buttons[i];
  }
  delete pattern_buttons[num_patterns];   // invisible button
  delete[] pattern_buttons;
  delete[] pattern_data;
}


const char*
PatternSelector::pattern_name ()
{
  return patterns[last_pattern_num].name;
} 


bool
PatternSelector::is_randomly_generated (int pattern_num)
{
  if (pattern_num == -1) pattern_num = last_pattern_num;
  return pattern_num < NUM_RANDOM_BUTTONS;
}

void 
PatternSelector::random ()
{
  int rnd = rand.get_int_range (0, num_patterns);
  unselect ();
  bool b = sensitive;
  sensitive = true;
  self_clicked = true;
  pattern_buttons[rnd]->clicked ();
  self_clicked = false;
  set_sensitive (b);
}


void 
PatternSelector::on_selection (int pattern_num)
{
  if (ignore_signal || !sensitive) return;
  
  // Re-select the previously selected pattern
  if (pattern_num == -1) {
    pattern_num = last_pattern_num;
    ignore_signal = true;
    pattern_buttons[pattern_num]->set_active (true);
  }
  // Ignore de-selection event
  if (!pattern_buttons[pattern_num]->get_active ()) return;
  
  // Scroll the selection into view
  if (is_realized () && pattern_buttons[pattern_num]->get_active ()) {
    Gtk::Adjustment* adjustment = get_vadjustment ();
    double button_size = adjustment->get_upper () / num_patterns;
    adjustment->clamp_page (button_size * pattern_num, button_size * (pattern_num + 1));
  }

  // Turn off the current random-pattern button if another button is selected
  if (last_pattern_num != pattern_num && is_randomly_generated (last_pattern_num) && hot) {
    hot = false;
    create_pixbuf (last_pattern_num);
  }
  // Turn on the selected random-pattern button, and randomize if it is selected while "hot"
  if (!ignore_signal && is_randomly_generated (pattern_num)) {
    if (hot || self_clicked) {
      build_random_dots (pattern_num);
      create_pixbuf (pattern_num);
    }
    timer.reset ();
    hot = true;
  }
  ignore_signal = false;

  Matrix m;
  get_matrix (m, pattern_num);
  last_pattern_num = pattern_num;
  pattern_changed_signal.emit ((PieceType*) m);
}


void
PatternSelector::unselect ()
{
  // Select the invisible button
  ignore_signal = true;
  pattern_buttons[num_patterns]->set_active (true);
  ignore_signal = false;
}


bool
PatternSelector::set_sensitive (bool b) 
{
  //get_child()->set_sensitive (b);
  sensitive = b;
  for (int i=0; i < num_patterns; i++) {
    pattern_buttons[i]->set_sensitive (b);
  }
  pattern_buttons[last_pattern_num]->set_sensitive (true);
}


void
PatternSelector::periodic () {
  if (is_randomly_generated () && hot) {
    create_pixbuf (last_pattern_num);
    pattern_buttons[last_pattern_num]->queue_draw ();
  }
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
