/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_PATTERN
#define PASANGEMAS_PATTERN 1
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/radiobutton.h>
#include <glibmm/random.h>
#include "pasang-emas.h"

namespace Pasang {

class PatternSelector : public Gtk::ScrolledWindow {
  public:
    // Type PieceType* below is actually Matrix
    sigc::signal<void, PieceType*>   pattern_changed_signal;

  private:
    Glib::Rand          rand;
    guint8*             pattern_data;
    Gtk::RadioButton**  pattern_buttons;
    int                 num_patterns;
    int                 last_pattern_num;
    bool                ignore_signal;
    bool                sensitive;
    Glib::Timer         timer;
    bool                hot;
    bool                self_clicked;
    
    // Matrix construction
    enum                {NUM_RANDOM_BUTTONS = 6};
    char                random_dots[NUM_RANDOM_BUTTONS][11*11 + 1];
    char                rand_char (const char*);
    char                read_code (char);
    void                build_random_day_and_night (int pattern_num);
    void                build_random_dots (int pattern_num);
    void                get_matrix (Matrix matrix, int pattern_num);
    
  public:
    PatternSelector ();
    virtual      ~PatternSelector ();
    const char*  pattern_name ();
    bool         is_randomly_generated (int pattern_num = -1);
    void         random();
    void         on_selection (int pattern_num = -1);    // -1 : use the previous pattern
    void         unselect ();
    virtual bool set_sensitive (bool);
    void         periodic ();
    
  private:
    Glib::RefPtr<Gdk::Pixbuf> create_pixbuf (int pattern_num);
    Glib::RefPtr<Gdk::Pixbuf> create_pixbuf (int pattern_num, Matrix);
};

}; //end namespace
#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
