/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "recording.h"
using namespace Pasang;

/**
 * Recording cannot be made when num_moves == -1
 */
Recording::Recording ()
{
  num_moves = -1;
  move_notations = new std::string[MAX_MOVE];
}

Recording::~Recording ()
{
  delete[] move_notations;
}
  
/**
 * Start recording
 */
void 
Recording::record(Board& board)
{
  initial_board = board;
  num_moves = 0;
}

/**
 * Only complete moves should be recorded. Moves leading to 
 * STAGE_RESOLVE or STAGE_RESOLVE_KAS_SELECTION shouldn't be submitted.
 */
void 
Recording::record (std::string& move)
{
  g_assert (num_moves != -1);
  g_assert (num_moves < MAX_MOVE);
  move_notations[num_moves] = move;
  num_moves++;
}

/**
 * The argument board is assumed to be the Board that 
 * is being recorded. If board's stage is either STAGE_RESOLVE
 * or STAGE_RESOLVE_KAS_SELECTION, we merely replay the
 * recording to undo the (unrecorded) partial move.
 */
void 
Recording::undo (Board& board)
{
  g_assert (num_moves != -1);
  if (num_moves == 0) return;
  
  const int stage = board.get_stage ();
  if (stage != STAGE_RESOLVE && stage != STAGE_RESOLVE_KAS_SELECTION) {
    num_moves--;
  }
  // Replay
  board = initial_board;
  for (int i=0; i < num_moves; i++) {
    board.generate_branches ();
    board.perform (move_notations[i]);
  }
}

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
