/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_THEME
#define PASANGEMAS_THEME 1
#include <gtkmm/window.h>
#include <gtkmm/dialog.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include "pasang-emas.h"

namespace Pasang {

class Theme {
  public:
    Glib::RefPtr<Gdk::Pixbuf>        kas_image[2];

  private:
    Glib::RefPtr<Gdk::Pixbuf>        board_image;
    Glib::RefPtr<Gdk::Pixbuf>        piece_image[PIECE_COUNT];
    Glib::RefPtr<Gdk::Pixbuf>        images[PIECE_COUNT + 1];  

    // TreeView for theme selection
    Gtk::Dialog*                     dialog;
    Gtk::TreeView*                   tree_view;
    Glib::RefPtr<Gtk::TreeSelection> tree_selection;
    Glib::RefPtr<Gtk::ListStore>     tree_model;
    std::vector<std::string>         theme_dirs;
    
    Glib::Rand    rand;
    Glib::Mutex   mutex;             // Make sure it is safe to apply requested theme
    int current_theme;
    int requested_theme;
    int current_zoom;
    int requested_zoom;
    int num_themes;

  public:
    Theme ();
    ~Theme ();
    void draw_board (Glib::RefPtr<Gdk::Window>, Glib::RefPtr<Gdk::GC>, int x, int y);
    void draw_piece (Glib::RefPtr<Gdk::Window>, Glib::RefPtr<Gdk::GC>, 
                     PieceType, int frame, int x, int y);
    bool detect_piece (PieceType, int frame, int x, int y);  // (x,y) is relative to piece centre
    int  count_frames (PieceType);
    
    // Image sizes
    int  board_image_width ();
    int  piece_image_width (PieceType);
    int  cell_width ();
    void show_dialog (Gtk::Window&);
    void random ();
    void zoom (int);
    bool change ();
    
  private:
    bool         change_theme ();
    bool         change_zoom ();
    void         create_dialog ();
    Gtk::Widget* create_theme_list ();
    void         on_selection ();
};

}; //end namespace
#endif    

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
