/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2008  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PASANGEMAS_WINDOW
#define PASANGEMAS_WINDOW 1
#include <gtkmm/box.h>
#include <gtkmm/button.h>
#include <gtkmm/notebook.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/dialog.h>
#include <gtkmm/textview.h>
#include <gtkmm/clipboard.h>
#include "brainsetter.h"
#include "recording.h"

namespace Pasang {

class Window : public Gtk::Window {
  private: 
    Gtk::Label      *statusbar;
    std::string      status;      // For debugging
    BoardGui        *board_gui;
    PatternSelector *pattern_selector;
    Recording       *recording;
    Brain           *brain;
    BrainSetter     *brain_setter;
    VirtualPlayer    virtual_player;
    VirtualPlayer    demo_player[2];
    Gtk::Label      *demo_player_labels[2];
    Gtk::Label      *round_label;
    Gtk::Dialog     *demo_dialog;
    Gtk::AboutDialog*about_dialog;
    long             request_id;  // To ensure we don't catch a move from aborted thinking
    int              num_scheduled_moves; // If more than one, then ignore the previously scheduled moves
                                          // Todo: find out how to cancel timer-scheduled events

    Glib::RefPtr<Gtk::UIManager>   ui_manager;
    Glib::RefPtr<Gtk::ActionGroup> action_group;
    Gtk::Label  *score_label[2]; // Display num_win[] and totalScore[]
    Gtk::Image  *kas_images[2];
    
    int     num_win[2];           // How many wins
    int     undo_num_win[2];
    int     num_rounds;            // How many rounds already played
    int     undo_num_rounds;

    enum {
      USER_VS_AI,
      AI_VS_AI,
    } mode;

  public:
    Window ();
    virtual ~Window ();
    virtual void on_realize ();
  
  private:
    void display_scores ();
    void stop_thinking ();
    void on_got_move ();
    void on_next_move (std::string*);
    void on_pattern_changed (const Matrix);
    bool on_timeout (int timer_number);
    void create_menus ();
    void on_action (int n);
    void copy ();
    bool show_demo_dialog ();
    void show_demo_subdialog (int p);
    void show_about_dialog ();
    void on_about_dialog_closed (int response);
};

}; //end namespace
#endif

// vim: tabstop=2: expandtab: textwidth=100: autoindent:
